@@ -365,13 +365,14 @@ void gControl::initAll(gContainer *parent)
 	_allow_show = false;
 	_direction = DIRECTION_DEFAULT;
 	_ignore_mouse = false;
+	_no_animation = false;
 
 	frame = border = widget = NULL;
 	_scroll = NULL;
 	hFree = NULL;
 	_grab = false;
 
-	_fg = _bg = COLOR_DEFAULT;
+	_fg = _bg = GB_COLOR_DEFAULT;
 #ifdef GTK3
 	_css = NULL;
 	_has_css_id = false;
@@ -1328,16 +1329,23 @@ void gControl::setCanFocus(bool vl)
 void gControl::setFocus()
 {
 	#if DEBUG_FOCUS
-	fprintf(stderr, "setFocus %s ?\n", name());
+	fprintf(stderr, "setFocus %s / activeControl = %s ?\n", name(), gApplication::activeControl() ? gApplication::activeControl()->name() : "NULL");
 	#endif
-	
+
 	if (_proxy)
 	{
+		#if DEBUG_FOCUS
+		fprintf(stderr, "-> proxy\n");
+		#endif
 		_proxy->setFocus();
 		return;
 	}
 
-	if (!canFocus() || hasFocus())
+	#if DEBUG_FOCUS
+	fprintf(stderr, "canFocus() = %d hasFocus() = %d\n", canFocus(), hasFocus());
+	#endif
+
+	if (!canFocus()) // || hasFocus())
 		return;
 	
 	gMainWindow *win = window();
@@ -1616,7 +1624,7 @@ void gControl::connectParent()
 
 gColor gControl::getFrameColor()
 {
-	return gDesktop::getColor(gDesktop::LIGHT_FOREGROUND);
+	return gDesktop::getColor(COLOR_LIGHT_FOREGROUND);
 }
 
 #ifdef GTK3
@@ -1629,62 +1637,10 @@ void gControl::drawBorder(cairo_t *cr)
 #else
 void gControl::drawBorder(GdkEventExpose *e)
 {
-	GdkWindow *win;
-	GtkShadowType shadow;
-	gint x, y, w, h;
-	cairo_t *cr;
-	GtkWidget *wid;
-	GtkAllocation a;
-
-	if (getFrameBorder() == BORDER_NONE)
-		return;
-
-	x = 0;
-	y = 0;
-	w = width();
-	h = height();
-
-	if (frame)
-		wid = frame;
-	else
-		wid = widget;
-
-	if (GTK_IS_LAYOUT(wid))
-		win = gtk_layout_get_bin_window(GTK_LAYOUT(wid));
-	else
-		win = gtk_widget_get_window(wid);
-
-	gtk_widget_get_allocation(wid, &a);
-	x = a.x;
-	y = a.y;
-
-	if (w < 1 || h < 1)
-		return;
-
-	switch (getFrameBorder())
-	{
-		case BORDER_PLAIN:
-
-			cr = gdk_cairo_create(win);
-			gt_cairo_draw_rect(cr, x, y, w, h, getFrameColor());
-			cairo_destroy(cr);
-			return;
-
-		case BORDER_SUNKEN: shadow = GTK_SHADOW_IN; break;
-		case BORDER_RAISED: shadow = GTK_SHADOW_OUT; break;
-		case BORDER_ETCHED: shadow = GTK_SHADOW_ETCHED_IN; break;
-
-		default:
-			return;
-	}
-
 	GdkRectangle clip;
 	gdk_region_get_clipbox(e->region, &clip);
-	GtkStyle *st = gtk_widget_get_style(widget);
-	if (use_base)
-		gtk_paint_box(st, win, GTK_STATE_NORMAL, shadow, &clip, widget, "entry", x, y, w, h);
-	else
-		gtk_paint_shadow(st, win, GTK_STATE_NORMAL, shadow, &clip, widget, NULL, x, y, w, h);
+
+	gt_draw_border(frame ? frame : widget, &clip, getFrameBorder(), getFrameColor(), use_base);
 }
 #endif
 
@@ -2010,42 +1966,6 @@ void gControl::realize(bool draw_frame)
 	updateFont();
 }
 
-void gControl::realizeScrolledWindow(GtkWidget *wid, bool doNotRealize)
-{
-	_scroll = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
-
-#ifdef GTK3
-	PATCH_CLASS(_scroll, GTK_TYPE_SCROLLED_WINDOW)
-	PATCH_CLASS(wid, GTK_TYPE_TEXT_VIEW)
-#endif
-
-#ifdef GTK3
-		border = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-		gtk_widget_set_hexpand(wid, TRUE);
-#else
-		border = gtk_alignment_new(0, 0, 1, 1);
-#endif
-	gtk_widget_set_redraw_on_allocate(border, TRUE);
-	widget = wid;
-	frame = border;
-	//_no_auto_grab = true;
-
-	//gtk_container_add(GTK_CONTAINER(border), GTK_WIDGET(_scroll));
-	gtk_scrolled_window_set_policy(_scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type(_scroll, GTK_SHADOW_NONE);
-	gtk_container_add(GTK_CONTAINER(border), GTK_WIDGET(_scroll));
-	gtk_container_add(GTK_CONTAINER(_scroll), widget);
-
-	if (!doNotRealize)
-		realize(true);
-	else
-		registerControl();
-
-	updateFont();
-	
-	gtk_widget_show_all(border);
-}
-
 void gControl::updateBorder()
 {
 	int pad;
@@ -2152,7 +2072,7 @@ void gControl::setName(char *name)
 
 gColor gControl::defaultBackground() const
 {
-	return gDesktop::getColor(gDesktop::BACKGROUND, !isEnabled());
+	return gDesktop::getColor(COLOR_BACKGROUND, !isEnabled());
 }
 
 #ifdef GTK3
@@ -2226,13 +2146,13 @@ void gControl::updateStyleSheet(bool dirty)
 	if (!isReallyVisible() || !_style_dirty)
 		return;
 
-	bg = _no_background ? background() : COLOR_DEFAULT;
+	bg = _no_background ? background() : GB_COLOR_DEFAULT;
 	fg = foreground(); //realForeground();
 
 	css = g_string_new(NULL);
 	_css_node = NULL;
 	
-	if (bg != COLOR_DEFAULT || fg != COLOR_DEFAULT)
+	if (bg != GB_COLOR_DEFAULT || fg != GB_COLOR_DEFAULT)
 	{
 		setStyleSheetNode(css, getStyleSheetColorNode());
 		gt_css_add_color(css, bg, fg);
@@ -2262,11 +2182,11 @@ void gControl::updateStyleSheet(bool dirty)
 
 gColor gControl::realBackground(bool no_default)
 {
-	if (_bg != COLOR_DEFAULT)
+	if (_bg != GB_COLOR_DEFAULT)
 		return _bg;
 	
 	if (!no_default)
-		return COLOR_DEFAULT;
+		return GB_COLOR_DEFAULT;
 	
 	if (!use_base && parent())
 		return parent()->realBackground(true);
@@ -2290,12 +2210,12 @@ void gControl::setBackground(gColor color)
 
 gColor gControl::realForeground(bool no_default)
 {
-	if (_fg != COLOR_DEFAULT)
+	if (_fg != GB_COLOR_DEFAULT)
 		return _fg;
 	else if (pr)
 		return pr->realForeground(no_default);
 	else
-		return no_default ? gDesktop::getColor(gDesktop::FOREGROUND) : COLOR_DEFAULT;
+		return no_default ? gDesktop::getColor(COLOR_FOREGROUND) : GB_COLOR_DEFAULT;
 }
 
 void gControl::setRealForeground(gColor color)
@@ -2308,7 +2228,7 @@ void gControl::setForeground(gColor color)
 		return;
 	
 	_fg = color;
-	_fg_set = color != COLOR_DEFAULT;
+	_fg_set = color != GB_COLOR_DEFAULT;
 #ifdef GTK3
 	updateStyleSheet(true);
 #endif
@@ -2322,7 +2242,7 @@ gColor gControl::realBackground(bool no_default)
 	if (_bg_set)
 		return use_base ? get_gdk_base_color(widget, isEnabled()) : get_gdk_bg_color(widget, isEnabled());
 	else
-		return no_default ? defaultBackground() : COLOR_DEFAULT;
+		return no_default ? defaultBackground() : GB_COLOR_DEFAULT;
 }
 
 static void set_background(GtkWidget *widget, gColor color, bool use_base)
@@ -2345,7 +2265,7 @@ void gControl::setRealBackground(gColor color)
 void gControl::setBackground(gColor color)
 {
 	_bg = color;
-	_bg_set = color != COLOR_DEFAULT;
+	_bg_set = color != GB_COLOR_DEFAULT;
 
 	if (!_bg_set)
 	{
@@ -2363,7 +2283,7 @@ gColor gControl::realForeground(bool no_default)
 	else if (pr)
 		return pr->realForeground(no_default);
 	else
-		return no_default ? gDesktop::getColor(gDesktop::FOREGROUND) : COLOR_DEFAULT;
+		return no_default ? gDesktop::getColor(COLOR_FOREGROUND) : GB_COLOR_DEFAULT;
 }
 
 static void set_foreground(GtkWidget *widget, gColor color, bool use_base)
@@ -2382,7 +2302,7 @@ void gControl::setRealForeground(gColor color)
 void gControl::setForeground(gColor color)
 {
 	_fg = color;
-	_fg_set = color != COLOR_DEFAULT;
+	_fg_set = color != GB_COLOR_DEFAULT;
 
 	if (!_fg_set)
 	{
@@ -2533,7 +2453,7 @@ int gControl::scrollHeight()
 
 void gControl::setScrollBar(int vl)
 {
-	if (!_scroll)
+	if (!_scroll || vl == _scrollbar)
 		return;
 
 	_scrollbar = vl & 3;
@@ -2844,7 +2764,7 @@ void gControl::drawBackground(cairo_t *cr)
 {
 	gColor col= background();
 	
-	if (col == COLOR_DEFAULT)
+	if (col == GB_COLOR_DEFAULT)
 		return;
 	/*{
 		if (!gtk_widget_get_has_window(border))
@@ -2861,7 +2781,7 @@ void gControl::drawBackground(GdkEventExpose *e)
 {
 	GtkAllocation a;
 
-	if (background() == COLOR_DEFAULT)
+	if (background() == GB_COLOR_DEFAULT)
 		return;
 
 	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(border));
