@@ -60,7 +60,7 @@ public:
 	bool isDesignIgnore() const { return _design_ignore; }
 
 	bool hovered();
-	virtual long handle();
+	virtual uintptr_t handle();
 	
 	int left() const { return bufX; }
 	int x() const { return left(); }
@@ -96,6 +96,7 @@ public:
 	gCursor* cursor();
 	void setCursor(gCursor *vl);
 	virtual void updateCursor(GdkCursor *cursor);
+	void updateCurrentCursor();
 	
 	gControl *next();
 	gControl *previous();
@@ -144,11 +145,17 @@ public:
 	bool isInverted() const { return _inverted; }
 
 	bool isRightToLeft() const;
+
+	void setIgnoreMouse(bool v) { _ignore_mouse = v; }
+	bool isIgnoreMouse() const { return _ignore_mouse; }
+	
+	void setNoAnimation(bool v) { _no_animation = v; }
+	bool isNoAnimation() const { return _no_animation; }
 	
 	gColor background() const { return _bg; }
 	gColor foreground() const { return _fg; }
-	virtual void setBackground(gColor color = COLOR_DEFAULT);
-	virtual void setForeground(gColor color = COLOR_DEFAULT);
+	virtual void setBackground(gColor color = GB_COLOR_DEFAULT);
+	virtual void setForeground(gColor color = GB_COLOR_DEFAULT);
 	virtual gColor defaultBackground() const;
 	gColor realBackground(bool no_default = false);
 	gColor realForeground(bool no_default = false);
@@ -187,6 +194,7 @@ public:
 	
 	gControl *proxy() const { return _proxy; }
 	bool setProxy(gControl *proxy);
+	gControl *proxyFor() const { return _proxy_for; }
 
 	int scrollX();
 	int scrollY();
@@ -268,7 +276,6 @@ public:
 	unsigned _old_tracking : 1;            // real value when Tracking is false
 	unsigned _bg_set : 1;                  // Have a private background
 	unsigned _fg_set : 1;                  // Have a private foreground
-	unsigned have_cursor : 1;              // If gApplication::setBusy() must update the cursor
 	unsigned use_base : 1;                 // Use base and text color for foreground and background
 
 	unsigned _visible : 1;                 // A control can be hidden if its width or height is zero
@@ -277,7 +284,7 @@ public:
 	unsigned _dirty_pos : 1;               // If the position of the widget has changed
 	unsigned _dirty_size : 1;              // If the size of the widget has changed
 	unsigned _inside : 1;                  // if we got an enter event, but not a leave event yet.
-	unsigned _has_border : 1;              // if the control has a border
+	unsigned _allow_show : 1;              // Allowed to be visible (after the first resize)
 
 	unsigned _locked : 4;                  // For locking events
 	unsigned frame_border : 4;
@@ -301,8 +308,8 @@ public:
 	unsigned _minimum_size_set : 1;        // If minimum size has been computed
 	unsigned _direction : 2;               // Text direction
 
-	unsigned _allow_show : 1;              // Allowed to be visible (after the first resize)
-	
+	unsigned _ignore_mouse : 1;            // Ignore mouse events (for drawing area only)
+	unsigned _no_animation : 1;
 	
 #ifdef GTK3
 	unsigned _style_dirty : 1;             // If the style must be refreshed
@@ -317,10 +324,8 @@ public:
 	void setParent(gContainer *parent) { pr = parent; }
 	void initAll(gContainer *pr);
 	void realize(bool draw_frame = false);
-	void realizeScrolledWindow(GtkWidget *wid, bool doNotRealize = false);
 	void registerControl();
 	void updateGeometry(bool force = false);
-	bool mustUpdateCursor() { return mouse() != -1 || have_cursor || !parent(); }
 	void updateEventMask();
 	
 	bool hasInputMethod() { return _has_input_method; }
@@ -330,7 +335,6 @@ public:
 	virtual void updateBorder();
 	int getFrameBorder() const { return frame_border; }
 	void setFrameBorder(int border);
-	virtual void setBorder(bool b);
 	bool hasBorder() const;
 	int getFramePadding() const { return frame_padding; }
 	void setFramePadding(int padding);
