@@ -242,7 +242,7 @@ static void aux_return_date_info(void *_object, const char *key)
 			nnum=sscanf(datestr, "%4d%2d%2d%2d%2d%2d",&ds.year, &ds.month, &ds.day, &ds.hour, &ds.min, &ds.sec);
 			if (nnum == 6)
 			{
-				if (!GB.MakeDate(&ds,&ret))
+				if (!GB.MakeDate(&ds, &ret, true))
 					GB.ReturnDate(&ret);
 			}		
 		}
@@ -1037,7 +1037,11 @@ BEGIN_METHOD(PDFPAGE_select, GB_INTEGER X; GB_INTEGER Y; GB_INTEGER W; GB_INTEGE
 	h = VARGOPT(H, (int32_t)THIS->page->getMediaHeight());
 
 	dev = new TextOutputDev (NULL, true, 0, false, false);
+#if POPPLER_VERSION_24_11_0
+	gfx = THIS->page->createGfx(dev,72.0,72.0,0,false,true,-1, -1, -1, -1, NULL, NULL);
+#else
 	gfx = THIS->page->createGfx(dev,72.0,72.0,0,false,true,-1, -1, -1, -1, false, NULL, NULL);
+#endif
 
 	THIS->page->display(gfx);
 	dev->endPage();
@@ -1169,7 +1173,7 @@ BEGIN_PROPERTY(PDFPAGELINKDATA_page)
 
 END_PROPERTY
 
-BEGIN_PROPERTY (PDFPAGELINKDATA_type)
+BEGIN_PROPERTY(PDFPAGELINKDATA_type)
 
 	GB.ReturnInteger ( (int32_t)THIS->action->getKind() );
 
