@@ -709,7 +709,7 @@ static void _draw_border(QPainter *p, int frame, QWidget *w, QStyleOptionFrame &
 void CCONTAINER_draw_border(QPainter *p, char frame, QWidget *wid)
 {
 	QStyleOptionFrame opt;
-	opt.init(wid);
+	opt.initFrom(wid);
 	opt.rect = QRect(0, 0, wid->width(), wid->height());
 	_draw_border(p, frame, wid, opt);
 }
@@ -796,7 +796,7 @@ void MyFrame::paintEvent(QPaintEvent *e)
 	{
 		CWIDGET *window = CWidget::get(parentWidget());
 		GB_COLOR col = CWIDGET_get_background(window);
-		if (col != COLOR_DEFAULT)
+		if (col != GB_COLOR_DEFAULT)
 			painter.fillRect(e->rect(), CCOLOR_make(col));
 	}
 
