@@ -2,7 +2,7 @@
 
   cpaint_impl.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -53,16 +53,6 @@
 #include "csvgimage.h"
 #include "cpaint_impl.h"
 
-/*class ClipInfo
-{
-public:
-	QPainterPath *path;
-	GB_RECT *rect;
-
-	ClipInfo() { path = NULL; rect = NULL; }
-	~ClipInfo() { delete path; delete rect; }
-};*/
-
 typedef
 	struct {
 		QPainter *painter;
@@ -84,6 +74,9 @@ typedef
 #define PAINTER(d) EXTRA(d)->painter
 #define PATH(d) EXTRA(d)->path
 //#define CLIP(d) EXTRA(d)->clip
+
+#define DASH_ZERO 0.0009765625
+#define LINE_WIDTH_1 0.9999
 	
 static bool _internal_paint = false;
 
@@ -128,21 +121,21 @@ static bool init_painting(GB_PAINT *d, QPaintDevice *device)
 	PAINTER(d)->setRenderHints(QPainter::Antialiasing, true);
 	PAINTER(d)->setRenderHints(QPainter::TextAntialiasing, true);
 	PAINTER(d)->setRenderHints(QPainter::SmoothPixmapTransform, true);
-	if (MAIN_right_to_left)
-		PAINTER(d)->setLayoutDirection(Qt::RightToLeft);
+	PAINTER(d)->setLayoutDirection(Qt::LayoutDirectionAuto);
 
-	pen = PAINTER(d)->pen();
-	pen.setCapStyle(Qt::FlatCap);
-	pen.setJoinStyle(Qt::MiterJoin);
-	pen.setMiterLimit(10.0);
-	pen.setWidthF(1.0);
-	PAINTER(d)->setPen(pen);
-	
 	if (GB.Is(d->device, CLASS_Control))
 		col = CWIDGET_get_real_foreground((CWIDGET *)d->device);
 	else
 		col = 0;
 	
+	pen = PAINTER(d)->pen();
+	pen.setCapStyle(Qt::FlatCap);
+	pen.setJoinStyle(Qt::MiterJoin);
+	pen.setMiterLimit(10.0);
+	pen.setWidthF(LINE_WIDTH_1);
+	pen.setColor(CCOLOR_make(col));
+	PAINTER(d)->setPen(pen);
+
 	PAINTER(d)->setBrush(CCOLOR_make(col));
 	
 	return FALSE;
@@ -247,7 +240,7 @@ static int Begin(GB_PAINT *d)
 		d->area.height = wid->height();
 		return FALSE;
 	}
-	else if (GB.Is(device, CLASS_UserControl))
+	else if (GB.Is(device, CLASS_UserControl) || GB.Is(device, CLASS_UserContainer))
 	{
 		MyContainer *wid;
 
@@ -283,11 +276,7 @@ static int Begin(GB_PAINT *d)
 		if (init_painting(d, target))
 			return TRUE;
 		// Paint.W / $hPrinter.PaperWidth * 25.4  / $hPrinter.Resolution
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
-		QSizeF size = printer->printer->pageLayout().pageSize().size(QPageSize::Millimeter);
-#else		
-		QSizeF size = printer->printer->paperSize(QPrinter::Millimeter);
-#endif
+		QSizeF size = CPRINTER_get_page_size(printer);
 		//qDebug("d->area.width = %g / paper width = %g / resolution = %d", d->area.width, (floor((double)size.width() * 1E6) / 1E6), printer->printer->resolution());
 		d->fontScale = 25.4 * d->area.width / (floor((double)size.width() * 1E6) / 1E6) / printer->printer->resolution();
 		
@@ -400,15 +389,28 @@ static void Antialias(GB_PAINT *d, int set, int *antialias)
 		*antialias = PAINTER(d)->testRenderHint(QPainter::Antialiasing) ? 1 : 0;
 }
 
-static void set_painter_font(QPainter *p, QFont f)
+static void set_painter_font(QPainter *p, const QFont &f)
 {
-	p->setFont(f);	
+	QFont pf;
+
+	pf.setFamily(f.family());
+	pf.setPointSizeF(f.pointSizeF());
+	pf.setWeight(f.weight());
+	pf.setStyle(f.style());
+	pf.setUnderline(f.underline());
+	pf.setStrikeOut(f.strikeOut());
+
+	p->setFont(pf);
+
 	// Strange bug of QT. Sometimes the font does not apply (cf. DrawTextShadow)
-	if (f != p->font())
+		
+	/*if (pf != p->font())
 	{
-		f.fromString(f.toString());
-		p->setFont(f);
-	}
+		fprintf(stderr, "set_painter_font: %s / %s\n", TO_UTF8(f.toString()), TO_UTF8(p->font().toString()));
+		QFont f2;
+		f2.fromString(f.toString());
+		p->setFont(f2);
+	}*/
 }
 
 static void apply_font(QFont &font, void *object = 0)
@@ -430,7 +432,7 @@ static void Font(GB_PAINT *d, int set, GB_FONT *font)
 	{
 		if (*font)
 			f = QFont(*((CFONT *)(*font))->font);
-		else if ((GB.Is(d->device, CLASS_DrawingArea) || GB.Is(d->device, CLASS_UserControl)))
+		else if ((GB.Is(d->device, CLASS_DrawingArea) || GB.Is(d->device, CLASS_UserControl) || GB.Is(d->device, CLASS_UserContainer)))
 			f = (((CWIDGET *)d->device)->widget)->font();
 		
 		apply_font(f);
@@ -509,7 +511,7 @@ static void ResetClip(GB_PAINT *d)
 	PAINTER(d)->setClipping(false);
 }
 
-static void get_path_extents(QPainterPath *path, GB_EXTENTS *ext, const QTransform &transform)
+static void get_path_extents(QPainterPath *path, GB_EXTENTS *ext)
 {
 	if (!path)
 	{
@@ -517,7 +519,8 @@ static void get_path_extents(QPainterPath *path, GB_EXTENTS *ext, const QTransfo
 		return;
 	}
 
-	QRectF rect = transform.inverted().mapRect(path->boundingRect());
+	//QRectF rect = transform.inverted().mapRect(path->boundingRect());
+	QRectF rect = path->boundingRect();
 
 	ext->x1 = (float)rect.left();
 	ext->y1 = (float)rect.top();
@@ -537,7 +540,7 @@ static void ClipExtents(GB_PAINT *d, GB_EXTENTS *ext)
 	}
 	else*/
 	QPainterPath path = PAINTER(d)->clipPath();
-	get_path_extents(&path, ext, QTransform()); //PAINTER(d)->transform());
+	get_path_extents(&path, ext); //, QTransform()); //PAINTER(d)->transform());
 }
 
 static void Fill(GB_PAINT *d, int preserve)
@@ -587,7 +590,7 @@ static void Stroke(GB_PAINT *d, int preserve)
 
 static void PathExtents(GB_PAINT *d, GB_EXTENTS *ext)
 {
-	get_path_extents(PATH(d), ext, PAINTER(d)->transform());
+	get_path_extents(PATH(d), ext); //, PAINTER(d)->transform());
 }
 
 static int PathContains(GB_PAINT *d, float x, float y)
@@ -621,8 +624,6 @@ static void PathOutline(GB_PAINT *d, GB_PAINT_OUTLINE_CB cb)
 	}
 }
 
-#define DASH_ZERO 0.0009765625
-
 static void Dash(GB_PAINT *d, int set, float **dashes, int *count)
 {
 	QPen pen = PAINTER(d)->pen();
@@ -775,7 +776,7 @@ static void LineWidth(GB_PAINT *d, int set, float *value)
 	QPen pen = PAINTER(d)->pen();
 	if (set)
 	{
-		pen.setWidthF((qreal)*value);
+		pen.setWidthF((qreal)(*value == 1 ? LINE_WIDTH_1 : *value));
 		PAINTER(d)->setPen(pen);
 	}
 	else
@@ -815,7 +816,18 @@ static void Operator(GB_PAINT *d, int set, int *value)
 			case GB_PAINT_OPERATOR_DEST_ATOP: mode = QPainter::CompositionMode_DestinationAtop; break;
 			case GB_PAINT_OPERATOR_XOR: mode = QPainter::CompositionMode_Xor; break;
 			case GB_PAINT_OPERATOR_ADD: mode = QPainter::CompositionMode_Plus; break;
-			case GB_PAINT_OPERATOR_SATURATE: mode = QPainter::CompositionMode_Multiply; break;
+			case GB_PAINT_OPERATOR_SATURATE: mode = QPainter::CompositionMode_SourceOver; break;
+			case GB_PAINT_OPERATOR_MULTIPLY: mode = QPainter::CompositionMode_Multiply; break;
+			case GB_PAINT_OPERATOR_SCREEN: mode = QPainter::CompositionMode_Screen; break;
+			case GB_PAINT_OPERATOR_OVERLAY: mode = QPainter::CompositionMode_Overlay; break;
+			case GB_PAINT_OPERATOR_DARKEN: mode = QPainter::CompositionMode_Darken; break;
+			case GB_PAINT_OPERATOR_LIGHTEN: mode = QPainter::CompositionMode_Lighten; break;
+			case GB_PAINT_OPERATOR_COLOR_DODGE: mode = QPainter::CompositionMode_ColorDodge; break;
+			case GB_PAINT_OPERATOR_COLOR_BURN: mode = QPainter::CompositionMode_ColorBurn; break;
+			case GB_PAINT_OPERATOR_HARD_LIGHT: mode = QPainter::CompositionMode_HardLight; break;
+			case GB_PAINT_OPERATOR_SOFT_LIGHT: mode = QPainter::CompositionMode_SoftLight; break;
+			case GB_PAINT_OPERATOR_DIFFERENCE: mode = QPainter::CompositionMode_Difference; break;
+			case GB_PAINT_OPERATOR_EXCLUSION: mode = QPainter::CompositionMode_Exclusion; break;
 			case GB_PAINT_OPERATOR_OVER: default: mode = QPainter::CompositionMode_SourceOver;
 		}
 		PAINTER(d)->setCompositionMode(mode);
@@ -836,18 +848,38 @@ static void Operator(GB_PAINT *d, int set, int *value)
 			case QPainter::CompositionMode_DestinationAtop: *value = GB_PAINT_OPERATOR_DEST_ATOP; break;
 			case QPainter::CompositionMode_Xor: *value = GB_PAINT_OPERATOR_XOR; break;
 			case QPainter::CompositionMode_Plus: *value = GB_PAINT_OPERATOR_ADD; break;
-			case QPainter::CompositionMode_Multiply: *value = GB_PAINT_OPERATOR_SATURATE; break;
+			case QPainter::CompositionMode_Multiply: *value = GB_PAINT_OPERATOR_MULTIPLY; break;
+			case QPainter::CompositionMode_Screen: *value = GB_PAINT_OPERATOR_SCREEN; break;
+			case QPainter::CompositionMode_Overlay: *value = GB_PAINT_OPERATOR_OVERLAY; break;
+			case QPainter::CompositionMode_Darken: *value = GB_PAINT_OPERATOR_DARKEN; break;
+			case QPainter::CompositionMode_Lighten: *value = GB_PAINT_OPERATOR_LIGHTEN; break;
+			case QPainter::CompositionMode_ColorDodge: *value = GB_PAINT_OPERATOR_COLOR_DODGE; break;
+			case QPainter::CompositionMode_ColorBurn: *value = GB_PAINT_OPERATOR_COLOR_BURN; break;
+			case QPainter::CompositionMode_HardLight: *value = GB_PAINT_OPERATOR_HARD_LIGHT; break;
+			case QPainter::CompositionMode_SoftLight: *value = GB_PAINT_OPERATOR_SOFT_LIGHT; break;
+			case QPainter::CompositionMode_Difference: *value = GB_PAINT_OPERATOR_DIFFERENCE; break;
+			case QPainter::CompositionMode_Exclusion: *value = GB_PAINT_OPERATOR_EXCLUSION; break;
 			case QPainter::CompositionMode_SourceOver: default: *value = GB_PAINT_OPERATOR_OVER;
 		}
 	}
 }
 
 
+static void Opacity(GB_PAINT *d, int set, float *value)
+{
+	if (set)
+		PAINTER(d)->setOpacity(*value);
+	else
+		*value = (float)PAINTER(d)->opacity();
+}
+
+
 static void NewPath(GB_PAINT *d)
 {
 	PRESERVE_PATH(d, FALSE);
 }
 
+
 static void ClosePath(GB_PAINT *d)
 {
 	CHECK_PATH(d);
@@ -951,13 +983,14 @@ static void CurveTo(GB_PAINT *d, float x1, float y1, float x2, float y2, float x
 }
 
 static QStringList text_sl;
-static QVector<int> text_w;
-static int text_line;
+static QVector<float> text_w;
+static float text_line;
 
-static int get_text_width(QPainter *dp, QString &s)
+static float get_text_width(QPainter *p, QString &s)
 {
-	int w, width = 0;
+	float w, width = 0;
 	int i;
+	QFontMetricsF fm(p->font());
 
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
 	text_sl = s.split('\n', Qt::KeepEmptyParts);
@@ -970,9 +1003,9 @@ static int get_text_width(QPainter *dp, QString &s)
 	for (i = 0; i < (int)text_sl.count(); i++)
 	{
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
-		w = dp->fontMetrics().horizontalAdvance(text_sl[i]);
+		w = fm.horizontalAdvance(text_sl[i]);
 #else
-		w = dp->fontMetrics().width(text_sl[i]);
+		w = fm.width(text_sl[i]);
 #endif
 		if (w > width) width = w;
 		text_w[i] = w;
@@ -981,9 +1014,10 @@ static int get_text_width(QPainter *dp, QString &s)
 	return width;
 }
 
-static int get_text_height(QPainter *dp, QString &s)
+static int get_text_height(QPainter *p, QString &s)
 {
-	text_line = dp->fontMetrics().height();
+	QFontMetricsF fm(p->font());
+	text_line = fm.height();
 	return text_line * (1 + s.count('\n'));
 }
 
@@ -1063,7 +1097,7 @@ static void get_text_extents(GB_PAINT *d, bool rich, const char *text, int len,
 
 	p.end();
 
-	get_path_extents(&path, ext, QTransform());
+	get_path_extents(&path, ext); //, QTransform());
 	_draw_path = NULL;
 }
 
@@ -1352,11 +1386,21 @@ static void TransformDelete(GB_TRANSFORM *matrix)
 static void TransformInit(GB_TRANSFORM matrix, float xx, float yx, float xy, float yy, float x0, float y0)
 {
 	QTransform *t = (QTransform *)matrix;
-	QMatrix m((qreal)xx, (qreal)yx, (qreal)xy, (qreal)yy, (qreal)x0, (qreal)y0);
-	QTransform t2(m);
+	QTransform t2((qreal)xx, (qreal)yx, (qreal)xy, (qreal)yy, (qreal)x0, (qreal)y0);
 	*t = t2;
 }
 
+static void TransformGet(GB_TRANSFORM matrix, float values[6])
+{
+	QTransform *t = (QTransform *)matrix;
+	values[0] = t->m11();
+	values[1] = t->m12();
+	values[2] = t->m21();
+	values[3] = t->m22();
+	values[4] = t->m31();
+	values[5] = t->m32();
+}
+
 static void TransformTranslate(GB_TRANSFORM matrix, float tx, float ty)
 {
 	QTransform *t = (QTransform *)matrix;
@@ -1369,6 +1413,12 @@ static void TransformScale(GB_TRANSFORM matrix, float sx, float sy)
 	t->scale((qreal)sx, (qreal)sy);
 }
 
+static void TransformShear(GB_TRANSFORM matrix, float sh, float sv)
+{
+	QTransform *t = (QTransform *)matrix;
+	t->shear((qreal)sh, (qreal)sv);
+}
+
 static void TransformRotate(GB_TRANSFORM matrix, float angle)
 {
 	QTransform *t = (QTransform *)matrix;
@@ -1441,6 +1491,7 @@ GB_PAINT_DESC PAINT_Interface =
 	LineWidth,
 	MiterLimit,
 	Operator,
+	Opacity,
 	NewPath,
 	ClosePath,
 	Arc,
@@ -1479,8 +1530,10 @@ GB_PAINT_MATRIX_DESC PAINT_MATRIX_Interface =
 	TransformCopy,
 	TransformDelete,
 	TransformInit,
+	TransformGet,
 	TransformTranslate,
 	TransformScale,
+	TransformShear,
 	TransformRotate,
 	TransformInvert,
 	TransformMultiply,
@@ -1527,7 +1580,11 @@ MyPaintEngine::~MyPaintEngine() {}
 
 void MyPaintEngine::patchFeatures()
 {
+#if QT6
+	if (type() == Pdf)
+#else
 	if (type() == PostScript || type() == Pdf)
+#endif
 	{
 		QPaintEngine::PaintEngineFeatures f = QPaintEngine::AllFeatures;
 		f &= (QPaintEngine::PorterDuff | QPaintEngine::PerspectiveTransform
@@ -1651,6 +1708,10 @@ int MyPaintDevice::metric(PaintDeviceMetric m) const
 		case PdmDpiY: return d->logicalDpiY();
 		case PdmPhysicalDpiX: return d->physicalDpiX();
 		case PdmPhysicalDpiY: return d->physicalDpiY();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
+		case PdmDevicePixelRatio: return d->devicePixelRatio();
+		case PdmDevicePixelRatioScaled: return d->devicePixelRatioFScale();
+#endif
 		default: return 0;
 	}
 }
