@@ -118,7 +118,7 @@ static void make_executable(void)
 	// inside /tmp, and /tmp may be mounted with the "noexec" flag.
 	
 	if (stat(TEMP_EXEC, &info) || chmod(TEMP_EXEC, info.st_mode | S_IXUSR | S_IXGRP | S_IXOTH))
-		fprintf(stderr, "gba: warning: cannot change executable permissions\n");
+		fprintf(stderr, "gba" GAMBAS_VERSION_STRING ": warning: cannot change executable permissions\n");
 
 	FILE_set_owner(TEMP_EXEC, FILE_cat(FILE_get_dir(ARCH_project), ".project", NULL));
 
@@ -133,6 +133,9 @@ static void make_executable(void)
 		err = "Cannot create executable";
 		goto __ERROR;
 	}
+
+	if (ARCH_verbose)
+		printf("Total size: %" PRIu64 " bytes\n", (uint64_t)info.st_size);
 	
 	return;
 
@@ -311,14 +314,14 @@ void ARCH_exit(void)
 
 	fclose(arch_file);
 
-	make_executable();
-
 	if (ARCH_verbose)
 	{
 		print_separation();
-		printf("Writing archive format version %d:\n%s\n", ARCH_VERSION,ARCH_output);
+		printf("Writing archive format version %d\n%s\n", ARCH_VERSION, ARCH_output);
 	}
-	
+
+	make_executable();
+
 	/* Free everything */
 
 	for (i = 0; i < TABLE_count(arch_table); i++)
