@@ -222,8 +222,9 @@ const void *const GAMBAS_Api[] =
 	(void *)COMPARE_variant,
 
 	(void *)GB_SplitDate,
-	(void *)GB_MakeDate,
+	(void *)DATE_make_local,
 	(void *)DATE_from_time,
+	(void *)DATE_to_time,
 	(void *)DATE_timer,
 	(void *)DATE_from_string,
 
@@ -277,6 +278,7 @@ const void *const GAMBAS_Api[] =
 	(void *)GB_CollectionSet,
 	(void *)GB_CollectionGet,
 	(void *)GB_CollectionEnum,
+	(void *)GB_CollectionBrowse,
 
 	(void *)GB_HashTableNew,
 	(void *)HASH_TABLE_delete,
@@ -868,7 +870,7 @@ static bool raise_event(OBJECT *observer, void *object, int func_id, int nparam,
 		{
 			ERROR_hook();
 
-			if (EXEC_debug)
+			if (FLAG.debug)
 			{
 				DEBUG.Main(TRUE);
 				MAIN_exit(TRUE, 0);
@@ -1319,7 +1321,7 @@ char *GB_GetErrorMessage()
 
 void GB_Deprecated(const char *msg, const char *func, const char *repl)
 {
-	if (EXEC_debug)
+	if (FLAG.debug)
 	{
 		if (repl)
 			GB_Error("&1: &2 is deprecated. Use &3 instead", msg, func, repl);
@@ -2083,10 +2085,10 @@ DATE_SERIAL *GB_SplitDate(GB_VALUE *arg)
 	return DATE_split((VALUE *)arg);
 }
 
-bool GB_MakeDate(DATE_SERIAL *date, GB_DATE *arg)
+/*bool GB_MakeDate(DATE_SERIAL *date, GB_DATE *arg, bool local)
 {
-	return DATE_make(date, (VALUE *)arg);
-}
+	return DATE_make_local(date, (VALUE *)arg, local);
+}*/
 
 
 int GB_StringLength(const char *str)
@@ -2101,7 +2103,7 @@ bool GB_NumberToString(int local, double value, const char *format, char **str,
 		LOCAL_format_number
 		(
 			value,
-			format ? LF_USER : LF_GENERAL_NUMBER,
+			format ? GB_LF_USER : GB_LF_GENERAL_NUMBER,
 			format,
 			format ? strlen(format) : 0,
 			str, len, local != 0
@@ -2446,7 +2448,7 @@ void GB_BrowseProject(GB_BROWSE_PROJECT_CALLBACK func)
 	ARCHIVE *arch = NULL;
 
 	ARCHIVE_get_current(&arch);
-	if (!arch || (arch == ARCHIVE_main && !EXEC_arch))
+	if (!arch || (arch == ARCHIVE_main && !FLAG.arch))
 	{
 		_browse_project_func = func;
 		FILE_recursive_dir(PROJECT_path, project_file_found, NULL, 0, FALSE);
@@ -2455,7 +2457,7 @@ void GB_BrowseProject(GB_BROWSE_PROJECT_CALLBACK func)
 		ARCHIVE_browse(arch, func);
 }
 
-void GB_BrowseDirectory(const char *dir, GB_BROWSE_CALLBACK before, GB_BROWSE_CALLBACK after)
+void GB_BrowseDirectory(const char *dir, GB_BROWSE_DIRECTORY_CALLBACK before, GB_BROWSE_DIRECTORY_CALLBACK after)
 {
 	FILE_recursive_dir(dir, before, after, 0, FALSE);
 }
@@ -2477,22 +2479,22 @@ void *GB_DebugGetExec(void)
 
 void GB_DebugBreakOnError(bool b)
 {
-	EXEC_break_on_error = b;
+	FLAG.break_on_error = b;
 }
 
 void GB_DebugInside(bool b)
 {
-	EXEC_debug_inside = b;
+	FLAG.debug_inside = b;
 }
 
 void GB_DebugHold(void)
 {
-	EXEC_debug_hold = TRUE;
+	FLAG.debug_hold = TRUE;
 }
 
 bool GB_SystemDebug(void)
 {
-	return EXEC_debug;
+	return FLAG.debug;
 }
 
 void GB_SystemHasForked(void)
@@ -2501,10 +2503,10 @@ void GB_SystemHasForked(void)
 	FILE_init();
 	LOCAL_init();
 	
-	if (EXEC_profile)
+	if (FLAG.profile)
 		DEBUG.Profile.Cancel();
-	EXEC_profile = FALSE;
-	EXEC_profile_instr = FALSE;
+	FLAG.profile = FALSE;
+	FLAG.profile_instr = FALSE;
 
 	SIGNAL_has_forked();
 
