@@ -422,16 +422,16 @@ bool ARCHIVE_find_from_path(ARCHIVE **parch, const char **ppath)
 		while (strncmp(path, "../", 3) == 0)
 		{
 			path += 3;
-			if (*parch == NULL || *parch == ARCHIVE_main)
-				continue;
+			/*if (*parch == NULL || *parch == ARCHIVE_main)
+				continue;*/
 
 			while (i < STACK_frame_count)
 			{
 				class = STACK_frame[i].cp;
-				//fprintf(stderr, "[%d] %s / %s\n", i, class ? class->name : "NULL", class && class->component && class->component->archive ? class->component->archive->name : "NULL");
+				//fprintf(stderr, "%s [%d] %s / %s\n", path, i, class ? class->name : "NULL", class && class->component && class->component->archive ? class->component->archive->name : "NULL");
 				if (class)
 				{
-					if (!class->component)
+					if (!class->component && *parch)
 					{
 						*parch = NULL;
 						break;
@@ -451,12 +451,13 @@ bool ARCHIVE_find_from_path(ARCHIVE **parch, const char **ppath)
 		}
 	}
 
-	if (*parch == NULL && EXEC_arch)
+	if (*parch == NULL && FLAG.arch)
 		*parch = ARCHIVE_main;
 
-	//fprintf(stderr, "--> '%s' / %s\n", *parch ? (*parch)->name : "(null)", *ppath);
-
 	*ppath = path;
+	
+	//fprintf(stderr, "--> '%s' / %s\n", *parch ? (*parch)->name : "(null)", *ppath);
+	
 	return *parch == NULL;
 }
 
@@ -579,7 +580,7 @@ void ARCHIVE_read(ARCHIVE *arch, int pos, void *buffer, int len)
 }
 
 
-void ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *pattern, int attr)
+bool ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *pattern, int attr)
 {
 	ARCHIVE_FIND find;
 	char abs_path[16];
@@ -594,7 +595,7 @@ void ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *pattern, int
 	if (ARCHIVE_get(arch, &path, &find))
 	{
 		arch_dir = NULL;
-		return;
+		return FALSE;
 	}
 
 	// ?? "." means that we want to browse the archive.
@@ -603,7 +604,7 @@ void ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *pattern, int
 		// By calling FILE_dir_first() again with an absolute path, we are sure that next calls to
 		// FILE_dir_next() will never call ARCHIVE_dir_next().
 		FILE_dir_first(FILE_cat(PROJECT_path, path, NULL), pattern, attr);
-		return;
+		return TRUE;
 	}
 
 	if (pattern == NULL || !*pattern)
@@ -637,6 +638,8 @@ void ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *pattern, int
 	STRING_free(&arch_prefix);
 	if (abs_len)
 		arch_prefix = STRING_new(abs_path, abs_len);
+	
+	return FALSE;
 }
 
 
