@@ -90,7 +90,7 @@ bool ARCHIVE_exist(ARCHIVE *arch, const char *path);
 bool ARCHIVE_stat(ARCHIVE *arch, const char *path, FILE_STAT *info);
 bool ARCHIVE_is_dir(ARCHIVE *arch, const char *path);
 
-void ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *pattern, int attr);
+bool ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *pattern, int attr);
 bool ARCHIVE_dir_next(char **name, int *len, int attr);
 
 bool ARCHIVE_find_from_path(ARCHIVE **parch, const char **ppath);
