@@ -59,7 +59,7 @@ void *CARRAY_out_of_bounds()
 
 void CARRAY_static_array()
 {
-	GB_Error((char *)E_SARRAY);
+	GB_Error((char *)E_READONLY);
 }
 
 
@@ -436,6 +436,82 @@ static void check_size(CARRAY *_object, int size, int inc)
 }
 
 
+bool CARRAY_equals(CARRAY *a, CARRAY *b)
+{
+	static void *jump[16] = 
+	{
+		&&__N, &&__m, &&__m, &&__m, &&__m, &&__m, &&__m, &&__m,
+		&&__m, &&__s, &&__s, &&__m, &&__v, &&__N, &&__N, &&__N
+	};
+	
+	int i;
+	
+	if (a->type != b->type || a->count != b->count || a->n_dim != b->n_dim)
+		return FALSE;
+	
+	if (a->dim)
+	{
+		// No need to check the last dim as the count is the same
+		for (i = 0; i < a->n_dim; i++)
+		{
+			if (a->dim[i] != b->dim[i])
+				return FALSE;
+		}
+	}
+	
+	if (TYPE_is_object(a->type))
+		goto __o;
+	else
+		goto *jump[a->type];
+	
+__m:
+	return memcmp(a->data, b->data, a->count * a->size) == 0;
+
+__s:
+	{
+		char **pa = (char **)a->data;
+		char **pb = (char **)b->data;
+		
+		for (i = 0; i < a->count; i++)
+		{
+			if (!STRING_equal(pa[i], STRING_length(pa[i]), pb[i], STRING_length(pb[i])))
+				return FALSE;
+		}
+		return TRUE;
+	}
+
+__v:
+	{
+		VARIANT *pa = (VARIANT *)a->data;
+		VARIANT *pb = (VARIANT *)b->data;
+		
+		for (i = 0; i < a->count; i++)
+		{
+			if (!COMPARE_equals_variant(&pa[i], &pb[i]))
+				return FALSE;
+		}
+		return TRUE;
+	}
+
+__o:
+	{
+		void **pa = (void **)a->data;
+		void **pb = (void **)b->data;
+		
+		for (i = 0; i < a->count; i++)
+		{
+			if (!COMPARE_equals_object_same(pa[i], pb[i]))
+				return FALSE;
+		}
+		return TRUE;
+	}
+	
+__N:
+	return FALSE;
+}
+
+//-------------------------------------------------------------------------
+
 BEGIN_METHOD(Array_new, GB_INTEGER size)
 
 	TYPE type;
@@ -523,14 +599,16 @@ BEGIN_METHOD(Array_new, GB_INTEGER size)
 		
 		THIS->count = size;
 	}
-
 	
 END_METHOD
 
 
 BEGIN_PROPERTY(Array_Type)
 
-	GB_ReturnInteger(THIS->type);
+	if (TYPE_is_object(THIS->type))
+		GB_ReturnInteger(T_OBJECT);
+	else
+		GB_ReturnInteger(THIS->type);
 
 END_PROPERTY
 
@@ -559,7 +637,7 @@ BEGIN_PROPERTY(Array_ReadOnly)
 	else
 	{
 		bool read_only = VPROP(GB_BOOLEAN);
-		if (THIS->read_only == read_only)
+		if (THIS->read_only == !!read_only)
 			return;
 		
 		if (THIS->read_only)
@@ -633,7 +711,7 @@ static bool copy_remove(CARRAY *_object, int start, int length, bool copy, bool
 		{
 			data = ARRAY_insert_many(&array->data, 0, length);
 			array->count += length;
-			memmove(data, get_data(THIS, start), length * THIS->size);
+			memmove(data, CARRAY_get_data_unsafe(THIS, start), length * THIS->size);
 			borrow(array, 0, -1);
 		}
 		
@@ -1544,7 +1622,7 @@ BEGIN_METHOD(Array_String_join, GB_STRING sep; GB_STRING esc)
 	uint lsep = 1;
 	char *esc = "";
 	uint lesc = 0;
-	char escl, escr;
+	char escl = 0, escr = 0;
 	int i;
 	char **data = (char **)THIS->data;
 	char *p, *p2;
@@ -1675,8 +1753,8 @@ BEGIN_METHOD_VOID(CARRAY_reverse)
 	if (count > 1)
 	{
 		size = THIS->size;
-		pi = get_data(THIS, 0);
-		pj = get_data(THIS, count - 1);
+		pi = CARRAY_get_data_unsafe(THIS, 0);
+		pj = CARRAY_get_data_unsafe(THIS, count - 1);
 
 		do
 		{
@@ -1752,6 +1830,7 @@ BEGIN_METHOD(Array_Bounds_get, GB_INTEGER index)
 
 END_PROPERTY
 
+
 BEGIN_METHOD(Array_Byte_ToString, GB_INTEGER start; GB_INTEGER length)
 
 	int start, length;
@@ -1911,6 +1990,7 @@ BEGIN_METHOD_VOID(ArrayOfStruct_next)
 
 END_METHOD
 
+
 static void error_convert(CARRAY *array)
 {
 	OBJECT_UNREF(array);
@@ -1967,6 +2047,18 @@ static bool _convert(CARRAY *src, CLASS *class, VALUE *conv)
 	return FALSE;
 }
 
+
+BEGIN_METHOD(Array_Equals, GB_OBJECT array)
+
+	CARRAY *array = (CARRAY *)VARG(array);
+	
+	if (GB_CheckObject(array))
+		return;
+	
+	GB_ReturnBoolean(CARRAY_equals(THIS, array));
+
+END_METHOD
+
 #else
 
 #include "gbx_c_array.h"
@@ -2046,10 +2138,11 @@ GB_DESC NATIVE_BooleanArray[] =
 	GB_METHOD("Copy", "Boolean[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Boolean[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Boolean[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)b[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Boolean[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Boolean[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Boolean[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)b[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Boolean[];"),
 
 	GB_END_DECLARE
 };
@@ -2083,10 +2176,11 @@ GB_DESC NATIVE_ByteArray[] =
 	GB_METHOD("Copy", "Byte[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Byte[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Byte[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)c[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Byte[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Byte[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Byte[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)c[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Byte[];"),
 
 	GB_METHOD("ToString", "s", Array_Byte_ToString, "[(Start)i(Length)i]"),
 	GB_STATIC_METHOD("FromString", "Byte[]", Array_Byte_FromString, "(String)s"),
@@ -2123,10 +2217,11 @@ GB_DESC NATIVE_ShortArray[] =
 	GB_METHOD("Copy", "Short[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Short[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Short[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)h[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Short[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Short[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Short[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)h[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Short[];"),
 
 	GB_END_DECLARE
 };
@@ -2160,10 +2255,11 @@ GB_DESC NATIVE_IntegerArray[] =
 	GB_METHOD("Copy", "Integer[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Integer[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Integer[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)i[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Integer[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Integer[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Integer[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)i[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Integer[];"),
 
 	GB_END_DECLARE
 };
@@ -2198,10 +2294,11 @@ GB_DESC NATIVE_LongArray[] =
 	GB_METHOD("Copy", "Long[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Long[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Long[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)l[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Long[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Long[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Long[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)l[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Long[];"),
 
 	GB_END_DECLARE
 };
@@ -2249,10 +2346,11 @@ GB_DESC NATIVE_PointerArray[] =
 	GB_METHOD("Copy", "Pointer[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Pointer[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Pointer[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)p[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Pointer[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Pointer[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Pointer[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)p[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Pointer[];"),
 
 	GB_END_DECLARE
 };
@@ -2286,10 +2384,11 @@ GB_DESC NATIVE_StringArray[] =
 	GB_METHOD("Copy", "String[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "String[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "String[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)s[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "String[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "String[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "String[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)s[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)String[];"),
 
 	GB_END_DECLARE
 };
@@ -2324,10 +2423,11 @@ GB_DESC NATIVE_FloatArray[] =
 	GB_METHOD("Copy", "Float[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Float[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Float[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)f[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Float[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Float[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Float[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)f[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Float[];"),
 
 	GB_END_DECLARE
 };
@@ -2362,10 +2462,11 @@ GB_DESC NATIVE_SingleArray[] =
 	GB_METHOD("Copy", "Single[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Single[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Single[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)g[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Single[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Single[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Single[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)g[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Single[];"),
 
 	GB_END_DECLARE
 };
@@ -2400,10 +2501,11 @@ GB_DESC NATIVE_DateArray[] =
 	GB_METHOD("Copy", "Date[]", Array_Copy, "[(Start)i(Length)i]"),
 	GB_METHOD("Extract", "Date[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Date[]", Array_Extract, "(Start)i[(Length)i]"),
+	GB_METHOD("Fill", NULL, Array_Fill, "(Value)d[(Start)i(Length)i]"),
 	GB_METHOD("Sort", "Date[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Date[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "Date[]", CARRAY_reverse, NULL),
-	GB_METHOD("Fill", NULL, Array_Fill, "(Value)d[(Start)i(Length)i]"),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Date[];"),
 
 	GB_END_DECLARE
 };
@@ -2440,9 +2542,10 @@ GB_DESC NATIVE_ObjectArray[] =
 	GB_METHOD("Extract", "Object[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Object[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Fill", NULL, Array_Fill, "(Value)o[(Start)i(Length)i]"),
-	GB_METHOD("Reverse", "Object[]", CARRAY_reverse, NULL),
 	GB_METHOD("Sort", "Object[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Object[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
+	GB_METHOD("Reverse", "Object[]", CARRAY_reverse, NULL),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Object[];"),
 
 	GB_END_DECLARE
 };
@@ -2474,9 +2577,10 @@ GB_DESC NATIVE_VariantArray[] =
 	GB_METHOD("Extract", "Variant[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Delete", "Variant[]", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Fill", NULL, Array_Fill, "(Value)v[(Start)i(Length)i]"),
-	GB_METHOD("Reverse", "Variant[]", CARRAY_reverse, NULL),
 	GB_METHOD("Sort", "Variant[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "Variant[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
+	GB_METHOD("Reverse", "Variant[]", CARRAY_reverse, NULL),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)Variant[];"),
 
 	GB_END_DECLARE
 };
@@ -2517,6 +2621,7 @@ GB_DESC NATIVE_TemplateArray[ARRAY_TEMPLATE_NDESC] =
 	GB_METHOD("Sort", "*[]", Array_Sort, "[(Mode)i]"),
 	GB_METHOD("SortUsing", "*[]", Array_SortUsing, "(Order)Array;[(Mode)i]"),
 	GB_METHOD("Reverse", "*[]", CARRAY_reverse, NULL),
+	GB_METHOD("Equals", "b", Array_Equals, "(Array)*[];"),
 
 	GB_END_DECLARE
 };
