@@ -738,14 +738,14 @@ __l2s:
 
 __g2s:
 
-	LOCAL_format_number(value->_single.value, LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
+	LOCAL_format_number(value->_single.value, GB_LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
 	STRING_new_temp_value(value, addr, len);
 	BORROW(value);
 	return;
 
 __f2s:
 
-	LOCAL_format_number(value->_float.value, LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
+	LOCAL_format_number(value->_float.value, GB_LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
 	STRING_new_temp_value(value, addr, len);
 	BORROW(value);
 	return;
@@ -1471,18 +1471,18 @@ __LONG:
 __DATE:
 	{
 		const DATE_SERIAL *date = DATE_split(value);
-		LOCAL_format_date(date, DATE_SERIAL_has_no_date(date) ? LF_LONG_TIME : LF_STANDARD, NULL, 0, addr, len);
+		LOCAL_format_date(date, DATE_SERIAL_has_no_date(date) ? GB_LF_LONG_TIME : GB_LF_STANDARD, NULL, 0, addr, len);
 		return;
 	}
 
 __SINGLE:
 
-	LOCAL_format_number(value->_single.value, LF_SHORT_NUMBER, NULL, 0, addr, len, TRUE);
+	LOCAL_format_number(value->_single.value, GB_LF_SHORT_NUMBER, NULL, 0, addr, len, TRUE);
 	return;
 
 __FLOAT:
 
-	LOCAL_format_number(value->_float.value, LF_STANDARD, NULL, 0, addr, len, TRUE);
+	LOCAL_format_number(value->_float.value, GB_LF_STANDARD, NULL, 0, addr, len, TRUE);
 	return;
 
 __STRING:
@@ -1730,7 +1730,7 @@ void VALUE_convert_boolean(VALUE *value)
 	static const void *jump[16] =
 	{
 		&&__NR, &&__OK, &&__c2b, &&__h2b, &&__i2b, &&__l2b, &&__g2b, &&__f2b,
-		&&__d2b, &&__s2b, &&__s2b, &&__N, &&__v2, &&__func, &&__N, &&__n2b
+		&&__d2b, &&__s2b, &&__s2b, &&__p2b, &&__v2, &&__func, &&__N, &&__n2b
 	};
 
 	char *addr;
@@ -1789,6 +1789,12 @@ __s2b:
 		STRING_unref(&addr);
 	value->type = T_BOOLEAN;
 	return;
+	
+__p2b:
+
+	value->_integer.value = -(value->_pointer.value != NULL);
+	value->type = T_BOOLEAN;
+	return;
 
 __n2b:
 
@@ -2070,14 +2076,14 @@ __l2s:
 
 __g2s:
 
-	LOCAL_format_number(value->_single.value, LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
+	LOCAL_format_number(value->_single.value, GB_LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
 	STRING_new_temp_value(value, addr, len);
 	BORROW(value);
 	return;
 
 __f2s:
 
-	LOCAL_format_number(value->_float.value, LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
+	LOCAL_format_number(value->_float.value, GB_LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
 	STRING_new_temp_value(value, addr, len);
 	BORROW(value);
 	return;
@@ -2161,7 +2167,10 @@ __2v:
 
 	/* VALUE_put ne fonctionne pas avec T_STRING ! */
 	if (value->type != T_NULL)
+	{
+		value->_variant.value.data = 0; //TODO: Optimize that?
 		VALUE_put(value, &value->_variant.value, value->type);
+	}
 
 	value->_variant.vtype = value->type;
 	goto __TYPE;
