@@ -29,6 +29,7 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <poll.h>
 
 #include "gb_array.h"
 #include "gbx_exec.h"
@@ -40,13 +41,9 @@
 //#define DEBUG_TIMER 1
 //#define DEBUG_WATCH 1
 
-static fd_set read_fd;
-static fd_set write_fd;
+static struct pollfd *_poll = NULL;
 
-static WATCH_CALLBACK *watch_callback = NULL;
-//static short *watch_index = NULL;
-
-static int max_fd = 0;
+static WATCH_CALLBACK *_callback = NULL;
 
 static WATCH_TIMER *_timers = NULL;
 
@@ -55,6 +52,8 @@ static bool _must_delete_callback = FALSE;
 
 static struct timeval _last_time = { 0 };
 
+//-------------------------------------------------------------------------
+
 #ifdef DEBUG_TIMER
 double _debug_time;
 static double time_to_double(const struct timeval *t)
@@ -100,10 +99,11 @@ static struct timeval *time_now(void)
 
 	gettimeofday(&current, NULL);
 	if (current.tv_usec < 0 || current.tv_usec >= 1000000)
-		fprintf(stderr, "gbx3: warning: gettimeofday: tv_usec = %ld!\n", current.tv_usec);
+		fprintf(stderr, "gbx3: warning: gettimeofday: tv_usec = %" PRIu64 "!\n", (uint64_t)current.tv_usec);
 	return &current;
 }
 
+//-------------------------------------------------------------------------
 
 static void add_timer(GB_TIMER *timer, const struct timeval *timeout)
 {
@@ -261,14 +261,12 @@ static bool get_timeout(const struct timeval *wait, struct timeval *tv)
 	return FALSE;
 }
 
+//-------------------------------------------------------------------------
 
 void WATCH_init(void)
 {
-	FD_ZERO(&read_fd);
-	FD_ZERO(&write_fd);
-
-	ARRAY_create(&watch_callback);
-	//ARRAY_create(&watch_index);
+	ARRAY_create(&_poll);
+	ARRAY_create(&_callback);
 	ARRAY_create(&_timers);
 
 	#ifdef DEBUG_TIMER
@@ -279,42 +277,19 @@ void WATCH_init(void)
 
 void WATCH_exit(void)
 {
-	ARRAY_delete(&watch_callback);
-	//ARRAY_delete(&watch_index);
+	ARRAY_delete(&_poll);
+	ARRAY_delete(&_callback);
 	ARRAY_delete(&_timers);
 }
 
 
-static void watch_fd(int fd, int flag, bool watch)
-{
-	#if DEBUG_WATCH
-	fprintf(stderr, "watch_fd: %d for %s: %s\n", fd, flag == GB_WATCH_READ ? "read" : "write", watch ? "set" : "clear");
-	#endif
-
-	if (flag == GB_WATCH_READ)
-	{
-		if (watch)
-			FD_SET(fd, &read_fd);
-		else
-			FD_CLR(fd, &read_fd);
-	}
-	else if (flag == GB_WATCH_WRITE)
-	{
-		if (watch)
-			FD_SET(fd, &write_fd);
-		else
-			FD_CLR(fd, &write_fd);
-	}
-}
-
-
-static int watch_find_callback(int fd)
+static int find_callback(int fd)
 {
 	int i;
 
-	for (i = 0; i < ARRAY_count(watch_callback); i++)
+	for (i = 0; i < ARRAY_count(_callback); i++)
 	{
-		if (watch_callback[i].fd == fd)
+		if (_callback[i].fd == fd)
 			return i;
 	}
 
@@ -322,72 +297,51 @@ static int watch_find_callback(int fd)
 }
 
 
-static int find_max_fd(void)
-{
-	int i;
-	int max = -1;
-
-	for (i = 0; i < ARRAY_count(watch_callback); i++)
-	{
-		if (watch_callback[i].fd > max)
-			max = watch_callback[i].fd;
-	}
-
-	return max;
-}
-
-
-static WATCH_CALLBACK *watch_create_callback(int fd)
+static int watch_create_callback(int fd)
 {
 	int pos;
 	WATCH_CALLBACK *wcb;
+	struct pollfd *pfd;
 
 	#if DEBUG_WATCH
 	fprintf(stderr, "watch_create_callback: %d\n", fd);
 	#endif
 
-	pos = watch_find_callback(fd);
+	pos = find_callback(fd);
 	if (pos < 0)
 	{
-		#if DEBUG_WATCH
-		pos = ARRAY_count(watch_callback);
-		#endif
-		wcb = ARRAY_add_void(&watch_callback);
+		pos = ARRAY_count(_callback);
+		wcb = ARRAY_add_void(&_callback);
 		wcb->fd = fd;
+		pfd = ARRAY_add_void(&_poll);
+		pfd->fd = fd;
 	}
 	else
-		wcb = &watch_callback[pos];
+		wcb = &_callback[pos];
 
-	if (fd > max_fd)
-		max_fd = fd;
-		
 	#if DEBUG_WATCH
 	fprintf(stderr, "watch_create_callback: %d -> %d read = %p (%p) write = %p (%p)\n", fd, pos, wcb->callback_read, (void *)wcb->param_read, wcb->callback_write, (void *)wcb->param_write);
 	#endif
 	
-	//ensure_watch_index(fd);
-	//watch_index[fd] = pos;
-	
-	return wcb;
+	return pos;
 }
 
 
 static void watch_delete_callback(int pos)
 {
 	WATCH_CALLBACK *wcb;
-	int fd;
+	struct pollfd *pfd;
 	
-	wcb = &watch_callback[pos];
-	fd = wcb->fd,
+	wcb = &_callback[pos];
+	pfd = &_poll[pos];
 
 	#if DEBUG_WATCH
+	int fd = wcb->fd;
 	fprintf(stderr, "watch_delete_callback: %d (%d)\n", fd, _do_not_really_delete_callback);
 	#endif
 
 	wcb->fd = -1;
-	watch_fd(fd, GB_WATCH_READ, FALSE);
-	watch_fd(fd, GB_WATCH_WRITE, FALSE);
-	max_fd = find_max_fd();
+	pfd->fd = -1;
 	
 	if (_do_not_really_delete_callback)
 	{
@@ -398,7 +352,8 @@ static void watch_delete_callback(int pos)
 		return;
 	}
 		
-	ARRAY_remove(&watch_callback, pos);
+	ARRAY_remove(&_callback, pos);
+	ARRAY_remove(&_poll, pos);
 
 	#if DEBUG_WATCH
 	fprintf(stderr, "--> deleted\n");
@@ -408,7 +363,7 @@ static void watch_delete_callback(int pos)
 
 static void watch_delete_callback_from_fd(int fd)
 {
-	int pos = watch_find_callback(fd);
+	int pos = find_callback(fd);
 	if (pos >= 0)
 		watch_delete_callback(pos);
 }
@@ -416,20 +371,18 @@ static void watch_delete_callback_from_fd(int fd)
 
 void WATCH_watch(int fd, int type, void *callback, intptr_t param)
 {
+	int pos;
 	WATCH_CALLBACK *wcb;
-
-	if (fd < 0 || fd > FD_SETSIZE)
-	{
-		if (type != GB_WATCH_NONE)
-			ERROR_warning("trying to watch fd #%d", fd);
-		return;
-	}
+	struct pollfd *pfd;
 
 	if (type == GB_WATCH_NONE)
 		watch_delete_callback_from_fd(fd);
 	else
 	{
-		wcb = watch_create_callback(fd);
+		pos = watch_create_callback(fd);
+		wcb = &_callback[pos];
+		pfd = &_poll[pos];
+		
 		if (type == GB_WATCH_READ)
 		{
 			wcb->callback_read = callback;
@@ -442,15 +395,18 @@ void WATCH_watch(int fd, int type, void *callback, intptr_t param)
 		}
 
 		#if DEBUG_WATCH
-		fprintf(stderr, "add watch: %d -> %d read = %p (%p) write = %p (%p)\n", fd, watch_find_callback(fd), wcb->callback_read, (void *)wcb->param_read, wcb->callback_write, (void *)wcb->param_write);
+		fprintf(stderr, "add watch: %d -> %d read = %p (%p) write = %p (%p)\n", fd, find_callback(fd), wcb->callback_read, (void *)wcb->param_read, wcb->callback_write, (void *)wcb->param_write);
 		#endif
 
 		if (!wcb->callback_read && !wcb->callback_write)
 			watch_delete_callback_from_fd(fd);
 		else
 		{
-			watch_fd(fd, GB_WATCH_READ, wcb->callback_read != NULL);
-			watch_fd(fd, GB_WATCH_WRITE, wcb->callback_write != NULL);
+			pfd->events = 0;
+			if (wcb->callback_read)
+				pfd->events |= POLLIN;
+			if (wcb->callback_write)
+				pfd->events |= POLLOUT;
 		}
 	}
 }
@@ -460,15 +416,17 @@ void WATCH_transfer_watch(void)
 	WATCH_CALLBACK *wcb;
 	int i;
 
-	for (i = 0; i < ARRAY_count(watch_callback); i++)
+	for (i = 0; i < ARRAY_count(_callback); i++)
 	{
-		wcb = &watch_callback[i];
+		wcb = &_callback[i];
 		if (wcb->callback_read) HOOK(watch)(wcb->fd, GB_WATCH_READ, wcb->callback_read, wcb->param_read);
 		if (wcb->callback_write) HOOK(watch)(wcb->fd, GB_WATCH_WRITE, wcb->callback_write, wcb->param_write);
 	}
 	
-	ARRAY_delete(&watch_callback);
-	ARRAY_create(&watch_callback);
+	ARRAY_delete(&_callback);
+	ARRAY_create(&_callback);
+	ARRAY_delete(&_poll);
+	ARRAY_create(&_poll);
 }
 
 
@@ -488,7 +446,7 @@ void WATCH_transfer_timer(void)
 }
 
 
-static void raise_callback(fd_set *rfd, fd_set *wfd)
+static void raise_callback()
 {
 	int i;
 	WATCH_CALLBACK wcb;
@@ -498,23 +456,22 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 	#if DEBUG_WATCH
 	fprintf(stderr, "raise_callback: max_fd = %d\n", max_fd);
 	#endif
-	for (i = 0; i < ARRAY_count(watch_callback); i++)
+	for (i = 0; i < ARRAY_count(_callback); i++)
 	{
-		// We copy the callback structure, because the watch_callback array can change during the
+		// We copy the callback structure, because the _callback array can change during the
 		// execution of the callbacks.
 		
-		if (watch_callback[i].fd < 0)
+		if (_callback[i].fd < 0)
 			continue;
 		
-		wcb = watch_callback[i];
+		wcb = _callback[i];
 
 		#if DEBUG_WATCH
 		fprintf(stderr, "raise_callback: [%d] fd = %d read = %p (%p) write = %p (%p)\n", i, wcb.fd, wcb.callback_read, (void *)wcb.param_read, wcb.callback_write, (void *)wcb.param_write);
 		#endif
 		
-		if (FD_ISSET(wcb.fd, rfd))
+		if (_poll[i].revents & POLLIN)
 		{
-			FD_CLR(wcb.fd, rfd);
 			if (wcb.callback_read)
 			{
 				#ifdef DEBUG_WATCH
@@ -524,12 +481,11 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 			}
 		}
 
-		if (watch_callback[i].fd < 0)
+		if (_callback[i].fd < 0)
 			continue;
 		
-		if (FD_ISSET(wcb.fd, wfd))
+		if (_poll[i].revents & POLLOUT)
 		{
-			FD_CLR(wcb.fd, wfd);
 			if (wcb.callback_write)
 			{
 				#ifdef DEBUG_WATCH
@@ -548,11 +504,12 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 		fprintf(stderr, "do must delete callback\n");
 		#endif
 		i = 0;
-		while (i < ARRAY_count(watch_callback))
+		while (i < ARRAY_count(_callback))
 		{
-			if (watch_callback[i].fd < 0)
+			if (_callback[i].fd < 0)
 			{
-				ARRAY_remove(&watch_callback, i);
+				ARRAY_remove(&_callback, i);
+				ARRAY_remove(&_poll, i);
 			}
 			else
 				i++;
@@ -562,32 +519,20 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 }
 
 
-static int do_select(fd_set *rfd, fd_set *wfd, struct timeval *timeout)
+static int do_poll(struct timeval *timeout)
 {
-	int fd;
+	int npfd = ARRAY_count(_poll);
 
-	for (fd = max_fd; fd >= 0; fd--)
-	{
-		if (FD_ISSET(fd, &read_fd) || FD_ISSET(fd, &write_fd))
-			break;
-	}
-
-	if (fd < 0 && !timeout)
+	if (npfd == 0 && !timeout)
 		return 0;
 
-	max_fd = fd;
-
-	*rfd = read_fd;
-	*wfd = write_fd;
-
-	return select(max_fd + 1, rfd, wfd, NULL, timeout);
+	return poll(_poll, npfd, timeout ? timeout->tv_sec * 1000 + timeout->tv_usec / 1000 : -1);
 }
 
 static bool do_loop(struct timeval *wait)
 {
 	int ret;
 	struct timeval tv;
-	fd_set rfd, wfd;
 	bool something_done = FALSE;
 
 	if (EVENT_check_post())
@@ -598,10 +543,10 @@ static bool do_loop(struct timeval *wait)
 	#endif
 
 	if (get_timeout(wait, &tv))
-		ret = do_select(&rfd, &wfd, NULL);
+		ret = do_poll(NULL);
 	else
 	{
-		ret = do_select(&rfd, &wfd, &tv);
+		ret = do_poll(&tv);
 		something_done = ARRAY_count(_timers) > 0;
 	}
 
@@ -620,7 +565,7 @@ static bool do_loop(struct timeval *wait)
 
 	if (ret > 0)
 	{
-		raise_callback(&rfd, &wfd);
+		raise_callback();
 		something_done = TRUE;
 	}
 	else if (ret < 0)
@@ -687,35 +632,36 @@ void WATCH_wait(int wait)
 
 int WATCH_process(int fd_end, int fd_output, int fd_error, int timeout)
 {
-	fd_set rfd;
-	int ret, fd_max;
-	struct timeval tv;
-
-	fd_max = fd_end > fd_output ? fd_end : fd_output;
-	fd_max = fd_error > fd_max ? fd_error : fd_max;
+	struct pollfd pfd[3];
+	int i, n;
+	int ret;
 
+	if (timeout <= 0)
+		timeout = -1;
+	
+	n = 0;
+	if (fd_end >= 0)
+	{
+		pfd[n].fd = fd_end;
+		pfd[n].events = POLLIN;
+		n++;
+	}
+	if (fd_output >= 0)
+	{
+		pfd[n].fd = fd_output;
+		pfd[n].events = POLLIN;
+		n++;
+	}
+	if (fd_error >= 0)
+	{
+		pfd[n].fd = fd_error;
+		pfd[n].events = POLLIN;
+		n++;
+	}
+	
 	for(;;)
 	{
-		FD_ZERO(&rfd);
-		if (fd_end >= 0)
-			FD_SET(fd_end, &rfd);
-		if (fd_output >= 0)
-			FD_SET(fd_output, &rfd);
-		if (fd_error >= 0)
-			FD_SET(fd_error, &rfd);
-
-		if (timeout > 0)
-		{
-			tv.tv_sec = timeout / 1000;
-			tv.tv_usec = (timeout % 1000) * 1000;
-			ret = select(fd_max + 1, &rfd, NULL, NULL, &tv);
-			if (ret == 0)
-				break;
-		}
-		else
-		{
-			ret = select(fd_max + 1, &rfd, NULL, NULL, NULL);
-		}
+		ret = poll(pfd, n, timeout);
 
 		//fprintf(stderr, "WATCH_process: select -> %d [%d] / fd_end -> %d\n", ret, errno, FD_ISSET(fd_end, &rfd));
 		
@@ -727,9 +673,16 @@ int WATCH_process(int fd_end, int fd_output, int fd_error, int timeout)
 
 	ret = timeout > 0 ? WP_TIMEOUT : WP_NOTHING;
 	
-	if (fd_end >= 0 && FD_ISSET(fd_end, &rfd)) ret += WP_END;
-	if (fd_output >= 0 && FD_ISSET(fd_output, &rfd)) ret += WP_OUTPUT;
-	if (fd_error >= 0 && FD_ISSET(fd_error, &rfd)) ret += WP_ERROR;
+	for (i = 0; i < n; i++)
+	{
+		if (pfd[i].revents & POLLIN)
+		{
+			if (pfd[i].fd == fd_end) ret += WP_END;
+			else if (pfd[i].fd == fd_output) ret += WP_OUTPUT;
+			else if (pfd[i].fd == fd_error) ret += WP_ERROR;
+		}
+	}
+	
 	return ret;
 }
 
